@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2005, 2007, 2008, 2009, 2010, 2011 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2020 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,19 +24,19 @@ struct store_parse_closure
   int ack;
   int type;
   int isuid;
-  size_t *set;
-  int count;
+  mu_msgset_t msgset;
 };
   
 static int
 store_thunk (imap4d_parsebuf_t p)
 {
   struct store_parse_closure *pclos = imap4d_parsebuf_data (p);
-  char *msgset;
+  char *mstr;
   char *data;
   int status;
+  char *end;
   
-  msgset = imap4d_parsebuf_next (p, 1);
+  mstr = imap4d_parsebuf_next (p, 1);
   data = imap4d_parsebuf_next (p, 1);
 
   if (*data == '+')
@@ -69,44 +68,79 @@ store_thunk (imap4d_parsebuf_t p)
 	imap4d_parsebuf_exit (p, "Bogus data suffix");
     }
 
-  /* Get the message numbers in set[].  */
-  status = util_msgset (msgset, &pclos->set, &pclos->count, pclos->isuid);
-  switch (status)
-    {
-    case 0:
-      break;
-
-    case EINVAL:
-      /* See RFC 3501, section 6.4.8, and a comment to the equivalent code
-	 in fetch.c */
-      p->err_text = "Completed";
-      return RESP_OK;
-
-    default:
-      p->err_text = "Failed to parse message set";
-      return RESP_NO;
-    }      
-
-  if (p->token[0] != '(')
-    imap4d_parsebuf_exit (p, "Syntax error");
-  imap4d_parsebuf_next (p, 1);
+  status = mu_msgset_create (&pclos->msgset, mbox, MU_MSGSET_NUM);
+  if (status)
+    imap4d_parsebuf_exit (p, "Software error");
   
-  do
+  /* Get the message numbers in set[].  */
+  status = mu_msgset_parse_imap (pclos->msgset,
+				 pclos->isuid ? MU_MSGSET_UID : MU_MSGSET_NUM,
+				 mstr, &end);
+  if (status)
+    imap4d_parsebuf_exit (p, "Failed to parse message set");
+
+  if (strcmp (p->token, "NIL"))
     {
-      int t;
-      if (!util_attribute_to_type (p->token, &t))
-	pclos->type |= t;
+      if (p->token[0] != '(')
+	imap4d_parsebuf_exit (p, "Syntax error");
+      while (imap4d_parsebuf_next (p, 1) && p->token[0] != ')')
+	if (mu_imap_flag_to_attribute (p->token, &pclos->type))
+	  imap4d_parsebuf_exit (p, "Unrecognized flag");
     }
-  while (imap4d_parsebuf_next (p, 1) && p->token[0] != ')');
   return RESP_OK;
 }
 
+static int
+_do_store (size_t msgno, mu_message_t msg, void *data)
+{
+  struct store_parse_closure *pclos = data;
+  mu_attribute_t attr = NULL;
+      
+  mu_message_get_attribute (msg, &attr);
+	      
+  switch (pclos->how)
+    {
+    case STORE_ADD:
+      mu_attribute_set_flags (attr, pclos->type);
+      break;
+      
+    case STORE_UNSET:
+      mu_attribute_unset_flags (attr, pclos->type);
+      break;
+      
+    case STORE_SET:
+      mu_attribute_unset_flags (attr, 0xffffffff); /* FIXME */
+      mu_attribute_set_flags (attr, pclos->type);
+    }
+
+	  
+  if (pclos->ack)
+    {
+      io_sendf ("* %lu FETCH (", (unsigned long) msgno);
+      
+      if (pclos->isuid)
+	{
+	  size_t uid;
+	  int rc = mu_mailbox_translate (mbox, MU_MAILBOX_UID_TO_MSGNO,
+					 msgno, &uid);
+	  if (rc == 0)
+	    io_sendf ("UID %lu ", (unsigned long) uid);
+	}
+      io_sendf ("FLAGS (");
+      util_print_flags (attr);
+      io_sendf ("))\n");
+    }
+  /* Update the flags of uid table.  */
+  imap4d_sync_flags (msgno);
+  return 0;
+}
+
 int
 imap4d_store0 (imap4d_tokbuf_t tok, int isuid, char **ptext)
 {
   int rc;
   struct store_parse_closure pclos;
-
+  
   memset (&pclos, 0, sizeof pclos);
   pclos.ack = 1;
   pclos.isuid = isuid;
@@ -118,64 +152,24 @@ imap4d_store0 (imap4d_tokbuf_t tok, int isuid, char **ptext)
 			     ptext);
   if (rc == RESP_OK)
     {
-      size_t i;
-      
-      for (i = 0; i < pclos.count; i++)
-	{
-	  mu_message_t msg = NULL;
-	  mu_attribute_t attr = NULL;
-	  size_t msgno = isuid ? uid_to_msgno (pclos.set[i]) : pclos.set[i];
-      
-	  if (msgno)
-	    {
-	      mu_mailbox_get_message (mbox, msgno, &msg);
-	      mu_message_get_attribute (msg, &attr);
-	      
-	      switch (pclos.how)
-		{
-		case STORE_ADD:
-		  mu_attribute_set_flags (attr, pclos.type);
-		  break;
-		  
-		case STORE_UNSET:
-		  mu_attribute_unset_flags (attr, pclos.type);
-		  break;
-      
-		case STORE_SET:
-		  mu_attribute_unset_flags (attr, 0xffffffff); /* FIXME */
-		  mu_attribute_set_flags (attr, pclos.type);
-		}
-	    }
-	  
-	  if (pclos.ack)
-	    {
-	      io_sendf ("* %lu FETCH (", (unsigned long) msgno);
-	      
-	      if (isuid)
-		io_sendf ("UID %lu ", (unsigned long) msgno);
-	      io_sendf ("FLAGS (");
-	      util_print_flags (attr);
-	      io_sendf ("))\n");
-	    }
-	  /* Update the flags of uid table.  */
-	  imap4d_sync_flags (pclos.set[i]);
-	}
-
+      mu_msgset_foreach_message (pclos.msgset, _do_store, &pclos);
+    
       *ptext = "Completed";
     }
-
-  free (pclos.set);
+  
+  mu_msgset_free (pclos.msgset);
   
   return rc;
 }
 
 int
-imap4d_store (struct imap4d_command *command, imap4d_tokbuf_t tok)
+imap4d_store (struct imap4d_session *session,
+              struct imap4d_command *command, imap4d_tokbuf_t tok)
 {
   int rc;
-  char *err_text;
+  char *err_text = NULL;
   
   rc = imap4d_store0 (tok, 0, &err_text);
-  return io_completion_response (command, rc, "%s", err_text);
+  return io_completion_response (command, rc, "%s", mu_prstr (err_text));
 }
 
