@@ -864,16 +864,7 @@ mu_stream_readdelim (mu_stream_t stream, char *buf, size_t size,
 
   if (stream->buftype == mu_buffer_none)
     {
-      if (stream->readdelim)
-	{
-	  size_t nread;
-	  rc = stream->readdelim (stream, buf, size, delim, &nread);
-	  if (pread)
-	    *pread = nread;
-	  stream->offset += nread;
-	}
-      else
-	rc = _stream_readdelim (stream, buf, size, delim, pread);
+      rc = _stream_readdelim (stream, buf, size, delim, pread);
     }
   else
     {
@@ -952,14 +943,12 @@ mu_stream_getdelim (mu_stream_t stream, char **pbuf, size_t *psize,
 	  n = needed;
 	}
 
-      if (stream->readdelim)
-	rc = stream->readdelim (stream, lineptr + cur_len, n - cur_len, delim,
-				&rdn);
-      else if (stream->buftype != mu_buffer_none)
+      if (stream->buftype == mu_buffer_none)
+	rc = _stream_readdelim (stream, lineptr + cur_len, n - cur_len, delim,
+				&rdn); 
+      else
 	rc = _stream_scandelim (stream, lineptr + cur_len, n - cur_len, delim,
 				&rdn);
-      else
-	rc = mu_stream_read (stream, lineptr + cur_len, 1, &rdn);
 
       if (rc || rdn == 0)
 	break;
@@ -1115,14 +1104,18 @@ mu_stream_close (mu_stream_t stream)
   if (!(stream->flags & _MU_STR_OPEN))
     return MU_ERR_NOT_OPEN;
   
-  mu_stream_flush (stream);
-  /* Do close the stream only if it is not used by anyone else */
-  if (stream->ref_count > 1)
-    return 0;
-  _stream_event (stream, _MU_STR_EVENT_CLOSE, 0, NULL);
-  if (stream->close)
-    rc = stream->close (stream);
-  _stream_clrflag (stream, _MU_STR_OPEN);
+  rc = mu_stream_flush (stream);
+  if (rc == 0)
+    {
+      /* Do close the stream only if it is not used by anyone else */
+      if (stream->ref_count > 1)
+	return 0;
+      _stream_event (stream, _MU_STR_EVENT_CLOSE, 0, NULL);
+      if (stream->close)
+	rc = stream->close (stream);
+      if (rc == 0)
+	_stream_clrflag (stream, _MU_STR_OPEN);
+    }
   return rc;
 }
 
