@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2000, 2004, 2006-2007, 2010-2012, 2014-2017 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2020 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -177,6 +176,25 @@ _tcp_ioctl (mu_stream_t stream, int code, int opcode, void *ptr)
 	    }
 	}
       break;
+
+    case MU_IOCTL_TCPSTREAM:
+      switch (opcode)
+	{
+	case MU_IOCTL_TCP_GETSOCKNAME:
+	  if (!ptr)
+	    return EINVAL;
+	  if (!tcp->source_addr)
+	    {
+	      int rc = mu_sockaddr_from_socket (&tcp->source_addr, tcp->fd);
+	      if (rc)
+		return rc;
+	    }
+	  return mu_sockaddr_copy ((struct mu_sockaddr **)ptr,
+				   tcp->source_addr);
+	default:
+	  return EINVAL;
+	}
+      break;
       
     default:
       return ENOSYS;
