@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2001, 2005-2007, 2009-2012, 2014-2016 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2020 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -50,6 +49,7 @@ static struct format_record format_table[] = {
   { "mbox", &mu_mbox_record },
   { "mh",   &mu_mh_record },
   { "maildir", &mu_maildir_record },
+  { "dotmail", &mu_dotmail_record },
   { "pop",  &mu_pop_record },
   { "pops",  &mu_pops_record },
   { "imap", &mu_imap_record },
@@ -111,6 +111,7 @@ SCM_DEFINE_PUBLIC (scm_mu_register_format, "mu-register-format", 0, 0, 1,
 {
   int status;
 
+  SCM_VALIDATE_REST_ARGUMENT (rest);
   if (scm_is_null (rest))
     {
       status = register_format (NULL);
@@ -174,9 +175,11 @@ mu_scm_init ()
 
   _mu_scm_mailer = scm_from_locale_string ("sendmail:" PATH_SENDMAIL);
   scm_c_define ("mu-mailer", _mu_scm_mailer);
-
+  scm_c_export ("mu-mailer", NULL);
+  
   _mu_scm_debug = scm_from_int (0);
   scm_c_define ("mu-debug", _mu_scm_debug);
+  scm_c_export ("mu-debug", NULL);
 
   _mu_scm_package = scm_from_locale_string (PACKAGE);
   scm_c_define ("mu-package", _mu_scm_package);
@@ -211,6 +214,7 @@ mu_scm_init ()
   mu_scm_mime_init ();
   mu_scm_debug_port_init ();
   mu_scm_debug_init ();
+  mu_scm_filter_init ();
   
 #include "mu_scm.x"
   
