@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2002, 2003, 2005, 
-   2007, 2009 Free Software Foundation, Inc.
+   Copyright (C) 1999-2020 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "mail.h"
 
@@ -42,7 +39,7 @@ mail_followup (int argc, char **argv)
   n = get_cursor ();
   if (n == 0)
     {
-      util_error (_("No applicable message"));
+      mu_error (_("No applicable message"));
       return 1;
     }
 
@@ -74,14 +71,15 @@ mail_followup (int argc, char **argv)
      (or should it be cc?) */
   for (mp = msglist; mp; mp = mp->next)
     compose_header_set (&env, MU_HEADER_TO,
-			util_get_sender (mp->msg_part[0], 0),
+			util_get_sender (msgset_msgno (mp), 0),
 			COMPOSE_SINGLE_LINE);
 
   msgset_free (msglist);
 
-  fprintf (ofile, "To: %s\n", compose_header_get (&env, MU_HEADER_TO, ""));
-  fprintf (ofile, "Subject: %s\n\n",
-	   compose_header_get (&env, MU_HEADER_SUBJECT, ""));
+  mu_printf ("To: %s\n",
+		    compose_header_get (&env, MU_HEADER_TO, ""));
+  mu_printf ("Subject: %s\n\n",
+		    compose_header_get (&env, MU_HEADER_SUBJECT, ""));
 
   status = mail_send0 (&env, mu_isupper (argv[0][0]));
   compose_destroy (&env);
