@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2007, 2009-2012, 2014-2016 Free Software
-   Foundation, Inc.
+   Copyright (C) 1999-2020 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,9 +21,9 @@
 /* Global variables and constants*/
 mu_mailbox_t mbox;            /* Mailbox being operated upon */
 size_t total;                 /* Total number of messages in the mailbox */
-int interactive;              /* Is the session interactive */  
+int interactive;              /* Is the session interactive */
 int read_recipients;          /* Read recipients from the message (mail -t) */
-
+mu_url_t secondary_url;       /* URL of the mailbox given with the -f option */
 static mu_list_t command_list;/* List of commands to be executed after parsing
 				 command line */
 const char *program_version = "mail (" PACKAGE_STRING ")";
@@ -35,6 +34,13 @@ const char *program_version = "mail (" PACKAGE_STRING ")";
 int hint;
 char *file;
 char *user;
+
+int mime_option;
+int skip_empty_attachments;
+char *default_encoding;
+char *default_content_type;
+static char *content_name;
+static char *content_filename;
 
 static void
 cli_f_option (struct mu_parseopt *po, struct mu_option *opt, char const *arg)
@@ -46,7 +52,7 @@ static void
 cli_file_option (struct mu_parseopt *po, struct mu_option *opt, char const *arg)
 {
   if (arg)
-    file = mu_strdup (arg);  
+    file = mu_strdup (arg);
   hint |= HINT_FILE_OPTION;
 }
 
@@ -59,41 +65,41 @@ cli_command_option (struct mu_parseopt *po, struct mu_option *opt,
     case 'e':
       util_cache_command (&command_list, "setq mode=exist");
       break;
-      
+
     case 'p':
       util_cache_command (&command_list, "setq mode=print");
       break;
-      
+
     case 'r':
       util_cache_command (&command_list, "set return-address=%s", arg);
       break;
-      
+
     case 'q':
       util_cache_command (&command_list, "set quit");
       break;
-      
+
     case 't':
       read_recipients = 1;
       util_cache_command (&command_list, "set editheaders");
       util_cache_command (&command_list, "setq mode=send");
       break;
-      
+
     case 'H':
       util_cache_command (&command_list, "setq mode=headers");
       break;
-      
+
     case 'i':
       util_cache_command (&command_list, "set ignore");
       break;
-      
+
     case 'n':
       util_do_command ("set norc");
       break;
-      
+
     case 'N':
       util_cache_command (&command_list, "set noheader");
       break;
-      
+
     case 'E':
       util_cache_command (&command_list, "%s", arg);
       break;
@@ -103,17 +109,15 @@ cli_command_option (struct mu_parseopt *po, struct mu_option *opt,
       break;
 
     case 0:
-      mu_parseopt_error (po,
-			 _("--%s: option should have been recognized"),
+      mu_parseopt_error (po, _("--%s: option should have been recognized"),
 			 opt->opt_long);
       exit (po->po_exit_error);
-      
+
     default:
-      mu_parseopt_error (po,
-			 _("-%c: option should have been recognized"),
+      mu_parseopt_error (po, _("-%c: option should have been recognized"),
 			 opt->opt_short);
       exit (po->po_exit_error);
-    }      
+    }
 }
 
 static void
@@ -134,9 +138,46 @@ cli_append (struct mu_parseopt *po, struct mu_option *opt, char const *arg)
 static void
 cli_attach (struct mu_parseopt *po, struct mu_option *opt, char const *arg)
 {
+  int fd = -1;
+
   hint |= HINT_SEND_MODE;
-  if (send_attach_file_default (arg))
-    exit (1);
+  if (strcmp (arg, "-") == 0)
+    {
+      arg = NULL;
+      fd = 0;
+    }
+  if (send_attach_file (fd, arg, content_filename, content_name,
+			default_content_type, default_encoding))
+    exit (po->po_exit_error);
+
+  mime_option = 1;
+  
+  free (content_name);
+  content_name = NULL;
+  free (content_filename);
+  content_filename = NULL;
+}
+
+static void
+cli_attach_fd (struct mu_parseopt *po, struct mu_option *opt, char const *arg)
+{
+  int rc, fd;
+
+  hint |= HINT_SEND_MODE;
+  rc = mu_str_to_c (arg, mu_c_int, &fd, NULL);
+  if (rc)
+    {
+      mu_parseopt_error (po, _("%s: bad descriptor"), arg);
+      exit (po->po_exit_error);
+    }
+
+  send_attach_file (fd, NULL, content_filename, content_name,
+		    default_content_type, default_encoding);
+
+  free (content_name);
+  content_name = NULL;
+  free (content_filename);
+  content_filename = NULL;
 }
 
 static struct mu_option mail_options[] = {
@@ -150,72 +191,95 @@ static struct mu_option mail_options[] = {
   { "exist",  'e', NULL,      MU_OPTION_DEFAULT,
     N_("return true if mail exists"),
     mu_c_string, NULL, cli_command_option },
-  
+
   { "byname", 'F', NULL,      MU_OPTION_DEFAULT,
     N_("save messages according to sender"),
     mu_c_string, NULL, cli_command_option },
-    
+
   { "headers", 'H', NULL,     MU_OPTION_DEFAULT,
     N_("write a header summary and exit"),
     mu_c_string, NULL, cli_command_option },
-  
+
   { "ignore",  'i', NULL,     MU_OPTION_DEFAULT,
     N_("ignore interrupts"),
     mu_c_string, NULL, cli_command_option },
-    
+
   { "norc",    'n', NULL,     MU_OPTION_DEFAULT,
     N_("do not read the system mailrc file"),
     mu_c_string, NULL, cli_command_option },
-    
+
   { "nosum",   'N', NULL,     MU_OPTION_DEFAULT,
     N_("do not display initial header summary"),
     mu_c_string, NULL, cli_command_option },
-    
+
   { "print",   'p', NULL,     MU_OPTION_DEFAULT,
     N_("print all mail to standard output"),
-    mu_c_string, NULL, cli_command_option },    
+    mu_c_string, NULL, cli_command_option },
   { "read",    0,   NULL,     MU_OPTION_ALIAS },
-  
+
   { "return-address", 'r', N_("ADDRESS"), MU_OPTION_DEFAULT,
     N_("use address as the return address when sending mail"),
     mu_c_string, NULL, cli_command_option },
-    
+
   { "quit",    'q', NULL,     MU_OPTION_DEFAULT,
     N_("cause interrupts to terminate program"),
     mu_c_string, NULL, cli_command_option },
-    
+
   { "subject", 's', N_("SUBJ"), MU_OPTION_DEFAULT,
     N_("send a message with the given SUBJECT"),
     mu_c_string, NULL, cli_subject },
-  
+
   { "to",      't', NULL,       MU_OPTION_DEFAULT,
     N_("read recipients from the message header"),
     mu_c_string, NULL, cli_command_option },
-    
+
   { "user",    'u', N_("USER"), MU_OPTION_DEFAULT,
     N_("operate on USER's mailbox"),
     mu_c_string, &user },
-  
+
   { "append",   'a', N_("HEADER: VALUE"), MU_OPTION_DEFAULT,
     N_("append given header to the message being sent"),
     mu_c_string, NULL, cli_append },
-    
+
+  { "alternative", 0, NULL, MU_OPTION_DEFAULT,
+    N_("force multipart/alternative content type"),
+    mu_c_bool, &multipart_alternative },
+
+  { "skip-empty-attachments", 0, NULL, MU_OPTION_DEFAULT,
+    N_("skip attachments with empty body"),
+    mu_c_bool, &skip_empty_attachments },
+
   { "exec" ,    'E', N_("COMMAND"), MU_OPTION_DEFAULT,
     N_("execute COMMAND"),
     mu_c_string, NULL, cli_command_option },
-  
+
   { "encoding",  0, N_("NAME"), MU_OPTION_DEFAULT,
     N_("set encoding for subsequent --attach options"),
     mu_c_string, &default_encoding },
-  
+
   { "content-type", 0, N_("TYPE"), MU_OPTION_DEFAULT,
     N_("set content type for subsequent --attach options"),
     mu_c_string, &default_content_type },
-  
+
+  { "content-name", 0, N_("NAME"), MU_OPTION_DEFAULT,
+    N_("set the Content-Type name parameter for the next --attach option"),
+    mu_c_string, &content_name },
+  { "content-filename", 0, N_("NAME"), MU_OPTION_DEFAULT,
+    N_("set the Content-Disposition filename parameter for the next --attach option"),
+    mu_c_string, &content_filename },
+
   { "attach",  'A', N_("FILE"), MU_OPTION_DEFAULT,
     N_("attach FILE"),
     mu_c_string, NULL, cli_attach },
 
+  { "attach-fd",  0, N_("FD"), MU_OPTION_DEFAULT,
+    N_("attach from file descriptor FD"),
+    mu_c_string, NULL, cli_attach_fd },
+
+  { "mime",    'M',  NULL, MU_OPTION_DEFAULT,
+    N_("compose MIME messages"),
+    mu_c_bool, &mime_option },
+
   MU_OPTION_END
 }, *options[] = { mail_options, NULL };
 
@@ -244,9 +308,9 @@ static char *mail_capa[] = {
   "debug",
   "mailbox",
   "locking",
-  NULL 
+  NULL
 };
-			     
+
 static char *
 mail_cmdline (void *closure, int cont MU_ARG_UNUSED)
 {
@@ -255,8 +319,8 @@ mail_cmdline (void *closure, int cont MU_ARG_UNUSED)
 
   while (1)
     {
-      if (mailvar_get (NULL, "autoinc", mailvar_type_boolean, 0) == 0
-          && !mu_mailbox_is_updated (mbox))
+      if (mailvar_is_true (mailvar_name_autoinc)
+	  && !mu_mailbox_is_updated (mbox))
 	{
 	  mu_mailbox_messages_count (mbox, &total);
 	  page_invalidate (0);
@@ -264,14 +328,14 @@ mail_cmdline (void *closure, int cont MU_ARG_UNUSED)
 	}
 
       rc = ml_readline (prompt);
-      
+
       if (ml_got_interrupt ())
 	{
 	  mu_error (_("Interrupt"));
 	  continue;
 	}
 
-      if (!rc && mailvar_get (NULL, "ignoreeof", mailvar_type_boolean, 0) == 0)
+      if (!rc && mailvar_is_true (mailvar_name_ignoreeof))
 	{
 	  mu_error (_("Use \"quit\" to quit."));
 	  continue;
@@ -330,7 +394,8 @@ static char *default_setup[] = {
   "set noinplacealiases",
   "set fromfield",
   "set headline=\"%>%a%4m %18f %16d %3L/%-5o %s\"",
-  
+  "unset folder",
+
   /* Start in mail reading mode */
   "setq mode=read",
   "set noquit",
@@ -342,7 +407,7 @@ static char *default_setup[] = {
 
   "set nullbody", /* Null message body is traditionally allowed */
   "set nullbodymsg=\"" N_("Null message body; hope that's ok") "\"",
-  
+
   /* These settings are not yet used */
   "set noonehop",
   "set nosendwait",
@@ -361,18 +426,18 @@ main (int argc, char **argv)
 {
   char *mode = NULL, *prompt = NULL, *p;
   int i, rc;
-  
+
   mu_stdstream_setup (MU_STDSTREAM_RESET_NONE);
   set_cursor (1);
 
   /* Native Language Support */
   MU_APP_INIT_NLS ();
-  
+
   /* Register the desired formats.  */
   mu_register_all_formats ();
 
-  mu_auth_register_module (&mu_auth_tls_module);  
-  
+  mu_auth_register_module (&mu_auth_tls_module);
+
   interactive = isatty (fileno (stdin));
 #ifdef HAVE_SIGACTION
   {
@@ -401,6 +466,7 @@ main (int argc, char **argv)
   setenv ("VISUAL", "vi", 0);
 
   /* set defaults for execution */
+  util_do_command ("setq PID=\"%lu\"", (unsigned long) getpid ());
   for (i = 0; i < sizeof (default_setup)/sizeof (default_setup[0]); i++)
     util_do_command ("%s", default_setup[i]);
 
@@ -409,27 +475,32 @@ main (int argc, char **argv)
     util_do_command ("set screen=%s", p);
   else
     util_do_command ("set screen=%d", util_getlines ());
-  
+
   p = getenv ("COLUMNS");
   if (p && p[strspn (p, "0123456789")] == 0)
     util_do_command ("set columns=%s", p);
   else
     util_do_command ("set columns=%d", util_getcols ());
-  
+
   /* Set the default mailer to sendmail.  */
-  mailvar_set ("sendmail",
+  mailvar_set (mailvar_name_sendmail,
 	       mu_strdup ("sendmail:" PATH_SENDMAIL), mailvar_type_string,
 	       MOPTF_OVERWRITE);
 
   /* argument parsing */
   mu_cli (argc, argv, &cli, mail_capa, NULL, &argc, &argv);
 
+  if (default_content_type || default_encoding)
+    mime_option = 1;
+  if (mime_option)
+    util_cache_command (&command_list, "set mime");
+
   if (read_recipients)
     {
       argv += argc;
       argc = 0;
     }
-  
+
   if ((hint & (HINT_SEND_MODE|HINT_FILE_OPTION)) ==
       (HINT_SEND_MODE|HINT_FILE_OPTION))
     {
@@ -453,21 +524,21 @@ main (int argc, char **argv)
 	      mu_error (_("-f requires at most one command line argument"));
 	      exit (1);
 	    }
-	  file = argv[0];
+	  file = mu_strdup (argv[0]);
 	}
       else if (user)
 	mu_asprintf (&file, "~/%s/mbox", user);
       else
-	file = "~/mbox";
+	file = mu_strdup ("~/mbox");
     }
   else if (argc || (hint & HINT_SEND_MODE))
     util_cache_command (&command_list, "setq mode=send");
   else if (user)
     mu_asprintf (&file, "%%%s", user);
 
-  
+
   /* read system-wide mail.rc and user's .mailrc */
-  if (mailvar_get (NULL, "rc", mailvar_type_boolean, 0) == 0)
+  if (mailvar_is_true (mailvar_name_rc))
     util_do_command ("source %s", SITE_MAIL_RC);
   if ((p = getenv ("MAILRC")) && *p)
     util_do_command ("source %s", getenv ("MAILRC"));
@@ -496,7 +567,7 @@ main (int argc, char **argv)
     }
 
   /* how should we be running? */
-  if (mailvar_get (&mode, "mode", mailvar_type_string, 1))
+  if (mailvar_get (&mode, mailvar_name_mode, mailvar_type_string, 1))
     exit (EXIT_FAILURE);
 
   /* Interactive mode */
@@ -512,10 +583,11 @@ main (int argc, char **argv)
 
       mu_argcv_string (argc, argv, &buf);
       rc = util_do_command ("mail %s", buf);
-      return mailvar_get (NULL, "mailx", mailvar_type_boolean, 0) ? rc : 0;
+      free (buf);
+      return mailvar_is_true (mailvar_name_mailx) ? 0 : rc;
     }
   /* Or acting as a normal reader */
-  else 
+  else
     {
       if ((rc = mu_mailbox_create_default (&mbox, file)) != 0)
 	{
@@ -528,6 +600,26 @@ main (int argc, char **argv)
 	  exit (EXIT_FAILURE);
 	}
 
+      if (file)
+	{
+	  /* Save URL of the file for further use */
+	  mu_url_t url;
+
+	  if (mu_mailbox_get_url (mbox, &url) == 0)
+	    {
+	      rc = mu_url_dup (url, &secondary_url);
+	      if (rc)
+		{
+		  mu_diag_funcall (MU_DIAG_ERROR, "mu_url_dup", NULL, rc);
+		  exit (EXIT_FAILURE);
+		}
+	    }
+	  /* Destroy the content of file prior to freeing it: it can contain
+	     password, although such usage is discouraged */
+	  memset (file, 0, strlen (file));
+	  free (file);
+	}
+
       if ((rc = mu_mailbox_open (mbox, MU_STREAM_RDWR|MU_STREAM_CREAT)) != 0)
 	{
 	  mu_url_t url = NULL;
@@ -562,33 +654,33 @@ main (int argc, char **argv)
 	  else if (strcmp (mode, "read"))
 	    {
 	      mu_error (_("Unknown mode `%s'"), mode);
-	      util_do_command ("quit");
+	      util_do_command (mailvar_name_quit);
 	      return 1;
 	    }
 	}
-      
+
       if (total == 0
 	  && (strcmp (mode, "read")
-	      || mailvar_get (NULL, "emptystart", mailvar_type_boolean, 0)))
-        {
-	  if (file)
-	    mu_printf (_("%s: 0 messages\n"), file);
+	      || !mailvar_is_true (mailvar_name_emptystart)))
+	{
+	  if (secondary_url)
+	    mail_summary (0, NULL);
 	  else
 	    mu_printf (_("No mail for %s\n"), user ? user : mail_whoami ());
-          return 1;
-        }
+	  return 1;
+	}
 
       /* initial commands */
-      if (mailvar_get (NULL, "header", mailvar_type_boolean, 0) == 0)
+      if (mailvar_is_true (mailvar_name_header))
 	{
 	  util_do_command ("summary");
 	  util_do_command ("headers");
 	}
 
-      mailvar_get (&prompt, "prompt", mailvar_type_string, 0);
+      mailvar_get (&prompt, mailvar_name_prompt, mailvar_type_string, 0);
       mail_mainloop (mail_cmdline, (void*) prompt, 1);
       mu_printf ("\n");
-      util_do_command ("quit");
+      util_do_command (mailvar_name_quit);
       return 0;
     }
   /* We should never reach this point */
@@ -629,8 +721,7 @@ mail_mainloop (char *(*input) (void *, int),
       if (do_history && !(mu_isspace (cmd[0]) || cmd[0] == '#'))
 	add_history (cmd);
 #endif
-      if (command)
-	free (command);
+      free (command);
     }
 }
 
@@ -639,7 +730,7 @@ mail_warranty (int argc MU_ARG_UNUSED, char **argv MU_ARG_UNUSED)
 {
   mu_printf (
 	 _("GNU Mailutils -- a suite of utilities for electronic mail\n"
-           "Copyright (C) 1999-2011 Free Software Foundation, Inc.\n\n"));
+	   "Copyright (C) 1999-2019 Free Software Foundation, Inc.\n\n"));
   mu_printf (
   _("   GNU Mailutils is free software; you can redistribute it and/or modify\n"
     "   it under the terms of the GNU General Public License as published by\n"
@@ -652,11 +743,10 @@ mail_warranty (int argc MU_ARG_UNUSED, char **argv MU_ARG_UNUSED)
     "   GNU General Public License for more details.\n"
     "\n"
     "   You should have received a copy of the GNU General Public License along\n"
-    "   with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>.\n" 
+    "   with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>.\n"
     "\n"
     "\n"
 ));
 
   return 0;
 }
-
