@@ -482,7 +482,7 @@ static const yytype_uint16 yyrline[] =
        0,    75,    75,    78,    82,    86,    90,    94,    98,   102,
      108,   109,   113,   119,   125,   129,   135,   136,   142,   148,
      151,   166,   175,   190,   202,   205,   211,   212,   216,   222,
-     223,   237,   243,   246,   252,   260,   270
+     223,   237,   243,   246,   252,   260,   264
 };
 #endif
 
@@ -1520,13 +1520,13 @@ yyreduce:
   case 30:
 #line 224 "msgset.y" /* yacc.c:1646  */
     {
-	     if ((yyvsp[0].mset)->npart == 1)
+	     if (msgset_length ((yyvsp[0].mset)) == 1)
 	       {
-		 (yyval.mset) = msgset_range ((yyvsp[-2].number), (yyvsp[0].mset)->msg_part[0]);
+		 (yyval.mset) = msgset_range ((yyvsp[-2].number), msgset_msgno ((yyvsp[0].mset)));
 	       }
 	     else
 	       {
-		 (yyval.mset) = msgset_range ((yyvsp[-2].number), (yyvsp[0].mset)->msg_part[0]-1);
+		 (yyval.mset) = msgset_range ((yyvsp[-2].number), msgset_msgno ((yyvsp[0].mset)) - 1);
 		 if (!(yyval.mset))
 		   YYERROR;
 		 msgset_append ((yyval.mset), (yyvsp[0].mset));
@@ -1573,27 +1573,21 @@ yyreduce:
   case 35:
 #line 261 "msgset.y" /* yacc.c:1646  */
     {
-	     if ((yyvsp[0].number) > message_count)
-	       {
-		 util_error_range ((yyvsp[0].number));
-		 YYERROR;
-	       }
-
 	     (yyval.mset) = msgset_make_1 ((yyvsp[0].number));
 	   }
-#line 1585 "msgset.c" /* yacc.c:1646  */
+#line 1579 "msgset.c" /* yacc.c:1646  */
     break;
 
   case 36:
-#line 271 "msgset.y" /* yacc.c:1646  */
+#line 265 "msgset.y" /* yacc.c:1646  */
     {
 	     (yyval.mset) = (yyvsp[-1].mset);
 	   }
-#line 1593 "msgset.c" /* yacc.c:1646  */
+#line 1587 "msgset.c" /* yacc.c:1646  */
     break;
 
 
-#line 1597 "msgset.c" /* yacc.c:1646  */
+#line 1591 "msgset.c" /* yacc.c:1646  */
       default: break;
     }
   /* User semantic actions sometimes alter yychar, and that requires
@@ -1821,7 +1815,7 @@ yyreturn:
 #endif
   return yyresult;
 }
-#line 275 "msgset.y" /* yacc.c:1906  */
+#line 269 "msgset.y" /* yacc.c:1906  */
 
 
 static int xargc;
@@ -1954,7 +1948,15 @@ msgset_parse (const int argc, char **argv, int flags, msgset_t **mset)
 	  rc = 1;
 	}
       else
-	*mset = result;
+	{
+	  if (result->crd[1] > message_count)
+	    {
+	      util_error_range (result->crd[1]);
+	      msgset_free (result);
+	      return 1;
+	    }
+	  *mset = result;
+	}
     }
   mu_opool_destroy (&tokpool);
   return rc;
@@ -1965,13 +1967,10 @@ msgset_free (msgset_t *msg_set)
 {
   msgset_t *next;
 
-  if (!msg_set)
-    return;
   while (msg_set)
     {
       next = msg_set->next;
-      if (msg_set->msg_part)
-	free (msg_set->msg_part);
+      free (msg_set->crd);
       free (msg_set);
       msg_set = next;
     }
@@ -1996,9 +1995,9 @@ msgset_make_1 (size_t number)
     return NULL;
   mp = mu_alloc (sizeof (*mp));
   mp->next = NULL;
-  mp->npart = 1;
-  mp->msg_part = mu_alloc (sizeof mp->msg_part[0]);
-  mp->msg_part[0] = number;
+  if (mu_coord_alloc (&mp->crd, 1))
+    mu_alloc_die ();
+  mp->crd[1] = number;
   return mp;
 }
 
@@ -2008,9 +2007,8 @@ msgset_dup (const msgset_t *set)
   msgset_t *mp;
   mp = mu_alloc (sizeof (*mp));
   mp->next = NULL;
-  mp->npart = set->npart;
-  mp->msg_part = mu_calloc (mp->npart, sizeof mp->msg_part[0]);
-  memcpy (mp->msg_part, set->msg_part, mp->npart * sizeof mp->msg_part[0]);
+  if (mu_coord_dup (set->crd, &mp->crd))
+    mu_alloc_die ();
   return mp;
 }
 
@@ -2030,7 +2028,7 @@ msgset_append (msgset_t *one, msgset_t *two)
     return two;
   for (last = one; last->next; last = last->next)
     {
-      msgset_remove (&two, last->msg_part[0]);
+      msgset_remove (&two, msgset_msgno (last));
     }
   last->next = two;
   return one;
@@ -2040,7 +2038,7 @@ int
 msgset_member (msgset_t *set, size_t n)
 {
   for (; set; set = set->next)
-    if (set->msg_part[0] == n)
+    if (msgset_msgno (set) == n)
       return 1;
   return 0;
 }
@@ -2054,13 +2052,14 @@ msgset_remove (msgset_t **pset, size_t n)
     {
       if (cur == NULL)
 	return;
-      if (cur->msg_part[0] == n)
+      if (msgset_msgno (cur) == n)
 	break;
       pnext = &cur->next;
       cur = cur->next;
     }
   *pnext = cur->next;
-  free (cur);
+  cur->next = NULL;
+  msgset_free (cur);
 }
 
 msgset_t *
@@ -2122,11 +2121,13 @@ msgset_expand (msgset_t *set, msgset_t *expand_by)
       {
 	mp = mu_alloc (sizeof *mp);
 	mp->next = NULL;
-	mp->npart = i->npart + j->npart;
-	mp->msg_part = mu_calloc (mp->npart, sizeof mp->msg_part[0]);
-	memcpy (mp->msg_part, i->msg_part, i->npart * sizeof i->msg_part[0]);
-	memcpy (mp->msg_part + i->npart, j->msg_part,
-		j->npart * sizeof j->msg_part[0]);
+	if (mu_coord_alloc (&mp->crd,
+			    mu_coord_length (i->crd) + mu_coord_length (j->crd)))
+	  mu_alloc_die ();
+	memcpy (&mp->crd[1], &i->crd[1],
+		mu_coord_length (i->crd) * sizeof i->crd[0]);
+	memcpy (&mp->crd[1] + mu_coord_length (i->crd), &j->crd[1],
+		mu_coord_length (j->crd) * sizeof j->crd[0]);
 
 	if (!first)
 	  first = mp;
@@ -2465,7 +2466,7 @@ check_set (msgset_t **pset)
       while (p)
 	{
 	  msgset_t *next = p->next;
-	  if (util_isdeleted (p->msg_part[0]))
+	  if (util_isdeleted (msgset_msgno (p)))
 	    {
 	      if ((flags & MSG_SILENT) && (prev || next))
 		{
@@ -2481,7 +2482,7 @@ check_set (msgset_t **pset)
 	      else
 		{
 		  mu_error (_("%lu: Inappropriate message (has been deleted)"),
-			      (unsigned long) p->msg_part[0]);
+			    (unsigned long) msgset_msgno (p));
 		  /* Delete entire set */
 		  delset = *pset;
 		  *pset = NULL;
@@ -2504,55 +2505,6 @@ check_set (msgset_t **pset)
   return rc;
 }
 
-static void
-revstr (char *s, char *e)
-{
-  while (s < e)
-    {
-      char t = *s;
-      *s++ = *--e;
-      *e = t;
-    }
-}
-
-char *
-msgset_part_str (const msgset_t *msgset, size_t npart)
-{
-  size_t len = 0;
-  size_t i;
-  char *result, *p;
-  
-  for (i = 0; i < npart; i++)
-    {
-      size_t n = msgset->msg_part[i];
-      do
-	len++;
-      while (n /= 10);
-      len++;
-    }
-
-  result = malloc (len);
-  p = result;
-  
-  for (i = 0; i < npart; i++)
-    {
-      char *s;
-      size_t n = msgset->msg_part[i];
-      if (i)
-	*p++ = '.';
-      s = p;
-      do
-	{
-	  unsigned x = n % 10;
-	  *p++ = x + '0';
-	}
-      while (n /= 10);
-      revstr (s, p);
-    }
-  *p = 0;
-
-  return result;
-}
 
 #if 0
 void
