@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001-2002, 2005, 2007, 2009-2012, 2014-2016 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2020 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -52,15 +51,16 @@ reply0 (msgset_t *mspec, mu_message_t msg, void *data)
   compose_env_t env;
   int status;
   char *str;
+  char const *rcpt;
 
-  set_cursor (mspec->msg_part[0]);
+  set_cursor (msgset_msgno (mspec));
   
   compose_init (&env);
 
   mu_message_get_header (msg, &hdr);
 
   compose_header_set (&env, MU_HEADER_TO,
-		      util_get_sender (mspec->msg_part[0], 0),
+		      util_get_sender (msgset_msgno (mspec), 0),
 		      COMPOSE_SINGLE_LINE);
 
   if (*(int*) data) /* reply starts with a lowercase */
@@ -83,8 +83,7 @@ reply0 (msgset_t *mspec, mu_message_t msg, void *data)
 	  const char *email;
 	  if (mu_address_sget_email (addr, i, &email) || email == NULL)
 	    continue;
-	  if ((mailvar_get (NULL, "metoo", mailvar_type_boolean, 0) == 0)
-	      || !mail_is_my_name (email))
+	  if (mailvar_is_true (mailvar_name_metoo) || !mail_is_my_name (email))
 	    compose_header_set (&env, MU_HEADER_TO,
 				email,
 				COMPOSE_SINGLE_LINE);
@@ -113,17 +112,15 @@ reply0 (msgset_t *mspec, mu_message_t msg, void *data)
 
   mu_printf ("To: %s\n",
 	   compose_header_get (&env, MU_HEADER_TO, ""));
-  str = compose_header_get (&env, MU_HEADER_CC, NULL);
-  if (str)
-    mu_printf ("Cc: %s\n", str);
+  rcpt = compose_header_get (&env, MU_HEADER_CC, NULL);
+  if (rcpt)
+    mu_printf ("Cc: %s\n", rcpt);
   mu_printf ("Subject: %s\n\n",
-		    compose_header_get (&env, MU_HEADER_SUBJECT, ""));
+	     compose_header_get (&env, MU_HEADER_SUBJECT, ""));
   
   make_in_reply_to (&env, msg);
   make_references (&env, msg);
-  status = mail_send0 (&env,
-		       mailvar_get (NULL, "byname", mailvar_type_boolean, 0)
-		                     == 0);
+  status = mail_send0 (&env, mailvar_is_true (mailvar_name_byname));
   compose_destroy (&env);
 
   return status;
@@ -133,7 +130,7 @@ int
 mail_reply (int argc, char **argv)
 {
   int lower = mu_islower (argv[0][0]);
-  if (mailvar_get (NULL, "flipr", mailvar_type_boolean, 0) == 0)
+  if (mailvar_is_true (mailvar_name_flipr))
     lower = !lower;
   return util_foreach_msg (argc, argv, MSG_NODELETED, reply0, &lower);
 }
