@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2009-2012, 2014-2016 Free Software Foundation, Inc.
+   Copyright (C) 2009-2020 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -34,7 +34,7 @@ PyObject *
 status_object (int status, PyObject *py_obj)
 {
   PyObject *py_ret = PyTuple_New (2);
-  PyTuple_SetItem (py_ret, 0, PyInt_FromLong (status));
+  PyTuple_SetItem (py_ret, 0, PyLong_FromLong (status));
   PyTuple_SetItem (py_ret, 1, py_obj);
   return _ro (py_ret);
 }
@@ -54,24 +54,19 @@ _mu_py_module_name (const char *nm)
 }
   
 PyObject *
-_mu_py_attach_module (const char *name, PyMethodDef *methods)
+_mu_py_attach_module (struct PyModuleDef *mdef)
 {
-  PyObject *module;
-  char *ns = _mu_py_module_name (name);
+  PyObject *module, *sysmodules;
+  char *ns = _mu_py_module_name (mdef->m_name);
   
-  if (!(module = PyImport_AddModule (ns)))
+  mdef->m_name = ns;
+  module = PyModule_Create (mdef);
+  if (!module)
     return NULL;
-
-  if (PyModule_AddObject (package, name, module))
-    return NULL;
-
-  Py_INCREF (module);
-
-  if (methods && !Py_InitModule (ns, methods))
-    return NULL;
-
-  PyList_Append (all, PyString_FromString (name));
-  free (ns);
+  sysmodules = PyImport_GetModuleDict ();
+  PyMapping_SetItemString (sysmodules, mdef->m_name, module);
+  PyList_Append (all, PyUnicode_FromString (mdef->m_name));
+  //  free (ns);
   return module;
 }
 
@@ -100,41 +95,49 @@ static PyMethodDef nomethods[] = {
   { NULL, NULL }
 };
 
-void
+PyObject *
 mu_py_attach_modules (void)
 {
-  package = Py_InitModule (PY_ROOT_NAME, nomethods);
-  if (!package)
-    return;
-
-  PyModule_AddStringConstant (package, "__version__", PY_PACKAGE_VERSION);
-  if (!PyModule_AddObject (package, "__all__", _ro (PyList_New (0))))
+  static struct PyModuleDef def = {
+    PyModuleDef_HEAD_INIT,
+    PY_ROOT_NAME,
+    NULL,
+    -1,
+    nomethods
+  };
+  package = PyModule_Create (&def);
+  if (package)
     {
-      all = PyObject_GetAttrString (package, "__all__");
-      if (!all || !PyList_Check (all))
-	return;
-    }
+      PyModule_AddStringConstant (package, "__version__", PY_PACKAGE_VERSION);
+      if (!PyModule_AddObject (package, "__all__", _ro (PyList_New (0))))
+	{
+	  all = PyObject_GetAttrString (package, "__all__");
+	  if (!all || !PyList_Check (all))
+	    return NULL;
+	}
 
-  _mu_py_attach_error ();
-  _mu_py_attach_address ();
-  _mu_py_attach_attribute ();
-  _mu_py_attach_auth ();
-  _mu_py_attach_body ();
-  _mu_py_attach_envelope ();
-  _mu_py_attach_errno ();
-  _mu_py_attach_header ();
-  _mu_py_attach_filter ();
-  _mu_py_attach_folder ();
-  _mu_py_attach_mailer ();
-  _mu_py_attach_mailbox ();
-  _mu_py_attach_mailcap ();
-  _mu_py_attach_message ();
-  _mu_py_attach_mime ();
-  _mu_py_attach_nls ();
-  _mu_py_attach_registrar ();
-  _mu_py_attach_secret ();
-  _mu_py_attach_sieve ();
-  _mu_py_attach_stream ();
-  _mu_py_attach_url ();
-  _mu_py_attach_util ();
+      _mu_py_attach_error ();
+      _mu_py_attach_address ();
+      _mu_py_attach_attribute ();
+      _mu_py_attach_auth ();
+      _mu_py_attach_body ();
+      _mu_py_attach_envelope ();
+      _mu_py_attach_errno ();
+      _mu_py_attach_header ();
+      _mu_py_attach_filter ();
+      _mu_py_attach_folder ();
+      _mu_py_attach_mailer ();
+      _mu_py_attach_mailbox ();
+      _mu_py_attach_mailcap ();
+      _mu_py_attach_message ();
+      _mu_py_attach_mime ();
+      _mu_py_attach_nls ();
+      _mu_py_attach_registrar ();
+      _mu_py_attach_secret ();
+      _mu_py_attach_sieve ();
+      _mu_py_attach_stream ();
+      _mu_py_attach_url ();
+      _mu_py_attach_util ();
+    }
+  return package;
 }
