@@ -1,5 +1,5 @@
 #  GNU Mailutils -- a suite of utilities for electronic mail
-#  Copyright (C) 2009, 2010, 2011 Free Software Foundation, Inc.
+#  Copyright (C) 2009-2020 Free Software Foundation, Inc.
 #
 #  This library is free software; you can redistribute it and/or
 #  modify it under the terms of the GNU Lesser General Public
@@ -46,7 +46,7 @@ class Attribute:
         elif value == False:
             self.unset_flags (flag)
         else:
-            raise TypeError, value
+            raise TypeError(value)
 
     def is_modified (self):
         return attribute.is_modified (self.attr)
