@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2002, 2005, 2007, 2009-2012, 2014-2018 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999-2020 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -51,8 +50,18 @@
 #include <mailutils/envelope.h>
 #include <mailutils/wordsplit.h>
 #include <mailutils/datetime.h>
+#include <mailutils/iterator.h>
+#include <mailutils/assoc.h>
 
 int msglist (mu_mailbox_t mbox, int show_all, int argc, char **argv, int **set, int *n);
 
+void *pattern_init (char const *pattern);
+int pattern_match (void *pat, char const *str);
+void pattern_free (void *pat);
+
+int message_body_stream (mu_message_t msg, int unix_header,
+			 char const *charset,
+			 mu_stream_t *pstr);
+
 #endif
 
